/*
This file is part of "Uakron Robot Project".

"Uakron Robot Project" is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

"Uakron Robot Project" is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "MiscMethods.h"
#include "BasicStamp2Library.h"

#define paddleChooserTextMsgSize 12

#define paddleChooserTextMaxMsgs 13

char paddleChooserText [paddleChooserTextMaxMsgs][paddleChooserTextMsgSize] = {
"Choose Tag ",
"B toggles  ",
"Start quits",
"0. Use: no ",
"1. Use: no ",
"2. Use: no ",
"3. Use: no ",
"4. Use: no ",
"5. Use: no ",
"6. Use: no ",
"7. Use: no ",
"8. Use: no ",
"9. Use: no "};


//constants which point to information in the
//string table
const int paddleMsgTitle1 = 0;
const int paddleMsgTitle2 = 1;
const int paddleMsgTitle3 = 2;
const int paddleMsgPaddleOffset = 3;
const int paddleMsgMaxPaddleIndex = 12;
const int paddleMsgMinPaddleIndex = 3;

//points to where in a string the 'no' or 'yes'
//will start
const int answerIndex = 8;

//information on the format of 
//"char foundPaddles[10]"
const int minPaddle = 0;
const int maximumPaddles = 9;
const int findPaddle = 0;
const int ignorePaddle = 1;
const int paddleFound = 2;


/**
 * Displays a menu to help the user decide which
 * RFID chips should be found on the playing field.
 *
 * @param foundPaddles - An array of 10 elements
 * (already initialized). If an element in the 
 * return array is '0' it means that it has to be
 * found. A '1' means it is not to be found.
 */

void choosePaddles(byte * foundPaddles)
{
    //Use this format of the returned array to your 
    //advantage. Use the array to keep track of what paddles 
    //have been found. Simply go in order through the array
    //and mark with a '1' when the paddle is found. When
    //all elements in the array are '1', the search is over.

    int index;
    
    for(index = 0; index < maximumPaddles; index++)
    {
        foundPaddles[index] = ignorePaddle;
    }
    
    formatAndSendToLcd(1,paddleChooserText[paddleMsgTitle1],paddleChooserTextMsgSize);
    formatAndSendToLcd(2,paddleChooserText[paddleMsgTitle2],paddleChooserTextMsgSize);
    formatAndSendToLcd(3,paddleChooserText[paddleMsgTitle3],paddleChooserTextMsgSize);
    
    int currentPaddle = 0;
    formatAndSendToLcd(4,paddleChooserText[currentPaddle + paddleMsgPaddleOffset],paddleChooserTextMsgSize);  

    byte nes;
    do
    {
        //to avoid the user pushing a button down
        //and the menu moving multiple times, wait
        //until user releases button before continuing
        //next iteration

        while(getButtons() != 0);
        
        nes = getButtons();
        
        if(nes & B_BUTTON)
        {
            if(foundPaddles[currentPaddle] == ignorePaddle)
            {
                foundPaddles[currentPaddle] = findPaddle;
                paddleChooserText[currentPaddle + paddleMsgPaddleOffset][answerIndex + 0] = 'y';
                paddleChooserText[currentPaddle + paddleMsgPaddleOffset][answerIndex + 1] = 'e';
                paddleChooserText[currentPaddle + paddleMsgPaddleOffset][answerIndex + 2] = 's';
                formatAndSendToLcd(4,paddleChooserText[currentPaddle + paddleMsgPaddleOffset],paddleChooserTextMsgSize); 
            }
            else
            {
                foundPaddles[currentPaddle] = ignorePaddle;
                paddleChooserText[currentPaddle + paddleMsgPaddleOffset][answerIndex + 0] = 'n';
                paddleChooserText[currentPaddle + paddleMsgPaddleOffset][answerIndex + 1] = 'o';
                paddleChooserText[currentPaddle + paddleMsgPaddleOffset][answerIndex + 2] = ' ';
                formatAndSendToLcd(4,paddleChooserText[currentPaddle + paddleMsgPaddleOffset],paddleChooserTextMsgSize); 
            }
        }
        
        else if(nes & UP_BUTTON)
        {
            currentPaddle++;
            if(currentPaddle >  maximumPaddles)
            {
                currentPaddle = minPaddle;
            }
            formatAndSendToLcd(4,paddleChooserText[currentPaddle + paddleMsgPaddleOffset],paddleChooserTextMsgSize); 
        }
        else if(nes & DOWN_BUTTON)
        {
            currentPaddle--;
            if(currentPaddle < minPaddle)
            {
                currentPaddle = maximumPaddles;
            }
            formatAndSendToLcd(4,paddleChooserText[currentPaddle + paddleMsgPaddleOffset],paddleChooserTextMsgSize); 
        }
        
    }
    while( (nes & START_BUTTON) == 0);
    
    return;
}



#define firmwareTestMaxMsgs 6
#define firmwareTestMsgSize 14
char firmwareTestText [firmwareTestMaxMsgs][firmwareTestMsgSize]= {
"Firmware Test",
"L/R: Steering",
"U/D: Motors  ",
"Start: Quit  "};

const int firmwareTestTitle = 0;
const int firmwareTestServos = 1;
const int firmwareTestMotors = 2;
const int firmwareTestSpeakers = 3;

const int distanceToMove = 10;

#define MAX_LEFT -60
#define MAX_RIGHT 60

void firmwareTest()
{
    clearLcd();
    formatAndSendToLcd(1,firmwareTestText[firmwareTestTitle],firmwareTestMsgSize);
    formatAndSendToLcd(2,firmwareTestText[firmwareTestServos],firmwareTestMsgSize);
    formatAndSendToLcd(3,firmwareTestText[firmwareTestMotors],firmwareTestMsgSize);
    formatAndSendToLcd(4,firmwareTestText[firmwareTestSpeakers],firmwareTestMsgSize);
    
    byte nes;
    int steer = 0;
    byte led = 0;
    setStraight();

    do
    {
        //to avoid the user pushing a button down
        //and the menu moving multiple times, wait
        //until user releases button before continuing
        //next iteration

        while(getButtons() != 0);
        
        nes = getButtons();
        
        if(nes & UP_BUTTON)
        {
            forwardMedium(5);
        }
        else if(nes & DOWN_BUTTON)
        {
            backwardMeduim(5);
        }

        else if(nes & LEFT_BUTTON)
        {
            steer -= 10;
            if(steer < MAX_LEFT)
            {
               steer = MAX_LEFT;
            }
            
            if(steer < 0)
            {
                turnLeft((byte) (0-steer));
            }
            else
            {
                turnRight((byte) steer);
            }
        }

        else if(nes & RIGHT_BUTTON)
        {
            steer += 10;
            if(steer > MAX_RIGHT)
            {
               steer = MAX_RIGHT;
            }
            
            if(steer < 0)
            {
                turnLeft((byte) (0-steer));
            }
            else
            {
                turnRight((byte) steer);
            }
        }

        else if(nes & B_BUTTON)
        {
            beep(5,5);
        }
        else if(nes & A_BUTTON)
        {
            led = ~led;

            setLeds(led,led);
        }

        else if(nes & SELECT_BUTTON)
        {
           playChargeTheme(); 
        }
        
    }
    while( (nes & START_BUTTON) == 0);
    
    return;
}


#define calibrationTestTextMaxMsgs 3
#define calibrationTestMsgSize 12
char calibrationTestText [calibrationTestTextMaxMsgs][calibrationTestMsgSize]= {
"Tune Test  ",
"B moves    ",
"Start exits"};

const int calibrationTestTitle = 0;
const int calibrationTestHowto1 = 1;
const int calibrationTestHowto2 = 2;


void calibrationTest()
{
    clearLcd();

    formatAndSendToLcd(1,calibrationTestText[calibrationTestTitle],calibrationTestMsgSize);
    formatAndSendToLcd(2,calibrationTestText[calibrationTestHowto1],calibrationTestMsgSize);
    formatAndSendToLcd(3,calibrationTestText[calibrationTestHowto2],calibrationTestMsgSize);
    
    byte nes;

    do
    {
        //to avoid the user pushing a button down
        //and the menu moving multiple times, wait
        //until user releases button before continuing
        //next iteration

        while(getButtons() != 0);
        
        nes = getButtons();
        
        if(nes & B_BUTTON)
        {
            setStraight();
            sleep(250);
            forwardMedium(100);
        }
        
    }
    while( (nes & START_BUTTON) == 0);
    
    stopMotor();
    return;
}


#define themesTextMaxMsgs 4
#define themesTextMsgSize 16
char themesText [themesTextMaxMsgs][themesTextMsgSize]= {
"Mario Theme    ",
"Charge Theme   ",
"Thanks to:     ",
"Christy Patusky"};

const int marioThemeTitle = 0;
const int chargeThemeTitle = 1;
const int themeSpecialThanks = 2;
const int composersName = 3;

void marioTheme()
{
    clearLcd();
    formatAndSendToLcd(1,themesText[marioThemeTitle],themesTextMsgSize);
    formatAndSendToLcd(3,themesText[themeSpecialThanks],themesTextMsgSize);
    formatAndSendToLcd(4,themesText[composersName],themesTextMsgSize);
    playMarioTheme();
    
    //sleep statement forces playing the theme to 
    //block until finished
    sleep(0);
}

void chargeTheme()
{
    clearLcd();
    formatAndSendToLcd(1,themesText[chargeThemeTitle],themesTextMsgSize);
    formatAndSendToLcd(3,themesText[themeSpecialThanks],themesTextMsgSize);
    formatAndSendToLcd(4,themesText[composersName],themesTextMsgSize);
    playChargeTheme();
    
    //sleep statement forces playing the theme to 
    //block until finished
    sleep(0);
}


#define maxPaddles 10
#define firstPercentBlock 5

/*
 * Takes the paddleToGet array and creates 2 strings formatted for output.
 *  - toGetMessageBuffer is a string that describes in numbers what paddles
 *    are still on the field.
 *  - percentageMessageBuffer is a string that displays what % of paddles
 *    (out of 10) are left on the field.
 */
void makeYetGetPrintString(char paddleToGetArray [10], int totalPaddles, char toGetMessageBuffer[21], char percentageMessageBuffer[21])
{
    setStringToAllNulls(toGetMessageBuffer);
    setStringToAllNulls(percentageMessageBuffer);

    char numberAsString [2];
    numberAsString[0] = '\0';

    int donePaddles = 0;
    int percentage;

    strcat(toGetMessageBuffer,"To Find: ");

    int index;
    for(index = 0; index < maxPaddles; index++)
    {
        if(paddleToGetArray[index] == paddleFound)
        {
            numberAsString[0] = '\0';
            itoa(index, numberAsString, 10);
            strcat(toGetMessageBuffer,numberAsString);
            donePaddles++; 
        }
    }

    if(totalPaddles <= 0)
    {
        totalPaddles = 1;
    }

    percentage = (donePaddles * 100) / (totalPaddles);

    if(percentage > 100)
    {
        percentage = 100;
    }


    strcat(percentageMessageBuffer,"%: 0 ");
    
    percentageMessageBuffer[16] = '1';
    percentageMessageBuffer[17] = '0';
    percentageMessageBuffer[18] = '0';

    for(index = 0; index < 10 && percentage >= 10; index++)
    {
        percentageMessageBuffer[index + firstPercentBlock] = 0xff;
        percentage -= 10;
    }

}


void setStringToAllNulls(char string[21])
{
    int index;
    for(index = 0; index < 22; index++)
    {
        string[index] = '\0';
    }
}

void reboot()
{
    asm volatile("jmp 0");
}


