/*
This file is part of "Uakron Robot Project".

"Uakron Robot Project" is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

"Uakron Robot Project" is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
 * This file contains various methods that will be used 
 * in various aspects of our project, including menu
 * driven tests and option menus.
 *
 * @author Quatre Horrace
 */

#ifndef __MISCMETHODS_H_
#define __MISCMETHODS_H_

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "BasicStamp2Library.h"
#include "LcdLibrary.h"
#include "NES_Remote.h"


/**
 * Displays a menu to help the user decide which
 * RFID chips should be found on the playing field.
 *
 * @param foundPaddles - An array of 10 elements
 * (already initialized). If an element in the 
 * return array is '0' it means that it has to be
 * found. A '1' means it is not to be found.
 */
void choosePaddles(byte * foundPaddles);


/*
 * Runs a basic test on the firmware. This consists
 * of testing the servos, the drive motors, and the
 * speakers.
 */
void firmwareTest();


/*
 * Runs a basic (manual) calibration test. This
 * consists of the robot moving forward for a 
 * distance, then stopping and waiting for 
 * more instructions via the LCD and nes remote.
 */
void calibrationTest();


/*
 * Plays the mario theme. Method blocks until song
 * is over.
 */
void marioTheme();

/*
 * Plays the charge theme. Method blocks until song
 * is over.
 */
void chargeTheme();

/*
 * Takes the paddleToGet array and creates 2 strings formatted for output.
 *  - toGetMessageBuffer is a string that describes in numbers what paddles
 *    are still on the field.
 *  - percentageMessageBuffer is a string that displays what % of paddles
 *    (out of 10) are left on the field.
 *
 * @param paddleToGetArray - array of the status on each paddle, whether
 *        used or not.
 * @param totalPaddles - the total number of paddles that we are searching for
 * @param toGetMessageBuffer[21]
 * @param percentageMessageBuffer[21]
 */
void makeYetGetPrintString(char paddleToGetArray [10], int totalPaddles, char toGetMessageBuffer[21], char percentageMessageBuffer[21]);

/*
 * Sets a string of 21 characters to all null
 * 
 * @param 
 */
void setStringToAllNulls(char string[21]);


/*
 * Soft-reboots the AVR processor
 */
void reboot();

#endif

