
/*
 * This file contains methods that will allow the
 * RFID reader to be accessed and queried.
 *
 * @author Branden Archer
 */


#ifndef __RFID_LIBRARY_H_
#define __RFID_LIBRARY_H_

#include <avr/io.h>
#include "BasicStamp2Library.h"

//These values are not hooked to anything. They are
//for your reference only.
#define FREQ 921600
#define BAUD 2400

#define byte char
/*
 * Initializes the communication between the RFID reader and
 * the AVR.
 */
void initRfidReader();

//Here is a table of the possible return values
//from readRfid():

//No errors occured, data is valid:
#define NO_ERROR                0x00

//No RFID was found below:
#define NO_RFID_ERROR           0x01

//Some bytes were received from Rfid Reader, but not all
//of them. A timeout occured while trying to
//get the rest of the data:
#define RFID_READ_TIMEOUT_ERROR 0x02


/*
 * Reads 10 bytes of data from the RFID paddle. This
 * data is returned in the data buffer given to the method.
 * A status byte is returned with the method. If the
 * byte == NO_ERROR then all went well and the data 
 * buffer is valid. If any error occured, the data
 * buffer is not written to. The errors are defined
 * above.
 * @args data - The data buffer where the 10 bytes will
 *              be stored.
 * @return status of the transmission. See the errors
 *         listed above for more details.
 */
byte readRfid(char data [10]);


#endif
